class Error(Exception):
    pass


class MovedURL(Error):
    """
    Exception to raise when the website has moved to a different location
    """
    pass


class HorriblyGoneWrongError(Error):
    """
    Exception to raise when there is something wrong gone which is difficult to comprehend
    """
    pass


class ResponseNotOKError(Error):
    """
    Exceptions to raise when the response is somthing other than 200
    """
    pass



from trello.providers import Provider
from trello.utility import get_soup
import requests


class Udemy(Provider):
    """Class to get all the courses from udemy"""

    def __init__(self, user_id):
        """Constructor to the Udemy class"""
        self.user_id = user_id
        self.url = "https://www.udemy.com/user/{}/".format(self.user_id)
        self.soup = get_soup(self.url)
        self.courses = []
        self.api_id = self.get_api_id()
        self.api_url = "https://www.udemy.com/api-2.0/users/{}/subscribed-profile-courses/".format(self.api_id)
        self.course_url = "https://www.udemy.com/api-2.0/courses/{}/public-curriculum-sections"
        self.get_cards_api()

    def get_cards_api(self):
        self.hit_api()

    def hit_api(self):
        # @TODO add headers
        r = requests.get(self.api_url)
        response = r.json()
        if response['next']:
            self.api_url = response['next']
            self.hit_api()
        self.courses.extend(response['results'])

    def get_api_id(self):
        t = self.soup.find('div', {'class': 'img-circle user-avatar__img'})
        id = t.attrs['style'].replace("background-image:url(", "").replace(")", "")
        return id.split('/')[-1].split('_')[0]

    def get_cards(self):
        self.get_all_pages()
        return self.courses

    def get_titles(self, soup):
        """Get all course titles from a page"""
        titles = []
        for course in soup.find_all('li', {'class': 'card'}):
            title = course.find('strong', {'class': 'details__name'}).text.strip().strip('\n').strip()
            url = course.find('a').get('href')
            img = course.find('img', {'class': 'img'}).get('src')
            titles.append({'title': title, 'url': 'https://www.udemy.com{}'.format(url), 'thumbnail': img})
        return titles

    def get_all_pages(self):
        """Methog to get all the pages from udemy course"""
        pages = self.get_max_pages()
        for i in range(1, int(pages) + 1):
            url = "https://udemy.com/user/{}/?subscribed_courses={}&key=subscribed_courses".format(self.get_user_id(),
                                                                                                   i)
            soup = get_soup(url)
            self.courses.extend(self.get_titles(soup))

    def get_user_id(self):
        """Method to get the user id from the url"""
        return self.url.strip('/').split('/')[-1]

    def get_max_pages(self):
        pages_text = self.soup.find('ul', {'class': 'pagination'}).find_all('li')[-2].text
        return pages_text.strip().strip('\n').strip()

    def get_curriculum(self, course_id):
        """
        Get the currliculum from any udemy course url
        """
        # @TODO add headers
        r = requests.get(self.course_url.format(str(course_id)))
        try:
            return [x['title'] for x in r.json()['results']]
        except KeyError:
            print(course_id)
            raise KeyError



import requests


class Trello(object):
    """
    A class to implement Trello Api
    """

    def __init__(self, trello_key, trello_token):
        """
        Constructor for `Trello` class
        """
        self.trello_key = trello_key
        self.trello_token = trello_token
        self.payload = {
            'key': self.trello_key,
            'token': self.trello_token
        }
        self.base_url = "https://api.trello.com/1"

    def _get_response(self, url):
        response = requests.request("GET", url, params=self.payload)
        if response.status_code != requests.codes.ok:
            return None
        return response.json()

    def get_lists(self, board_id):
        return self._get_response(self.base_url + '/boards/{}/lists'.format(board_id))

    def get_cards(self, list_id):
        return self._get_response(self.base_url + '/lists/{}/cards'.format(list_id))

    def create_checklist(self, card_id, name):
        """
        Create a checklist on the card with `card_id`
        return id of the checklist created else None
        """
        url = self.base_url + '/cards/{}/checklists'.format(card_id)
        payload = {
            'key': self.trello_key,
            'token': self.trello_token,
            'name': name
        }
        response = requests.request("POST", url, params=payload)
        if response.status_code != requests.codes.ok:
            return None
        return response.json()['id']

    def add_items_to_checklist(self, checklist_id, items):
        """
        Adds item to an already created checklists with `checklist_id`
        `items` should be an iterable.
        """
        # @TODO add exceptions
        url = self.base_url + '/checklists/{}/checkItems'.format(checklist_id)
        for name in items:
            payload = {
                'key': self.trello_key,
                'token': self.trello_token,
                'name': name
            }
            requests.request("POST", url, params=payload)

    def create_card(self, list_id, name, desc):
        """
        Create a card on the list with id `list_id`
        """
        url = self.base_url + '/cards'
        payload = {
            'key': self.trello_key,
            'token': self.trello_token,
            'idList': list_id,
            'name': name,
            'desc': desc
        }
        response = requests.request("POST", url, params=payload)
        if response.status_code != requests.codes.ok:
            return None
        return response.json()['id']

    def add_attachment_to_card(self, card_id, attachment_url, name):
        """
        Add an attachment to an existing card with `card_id`
        """
        url = self.base_url + '/cards/{}/attachments'.format(card_id)
        payload = {
            'key': self.trello_key,
            'token': self.trello_token,
            'id': card_id,
            'url': attachment_url,
            'name': name
        }
        response = requests.request("POST", url, params=payload)
        if response.status_code != requests.codes.ok:
            return None
        return response.json()['id']



import requests
from bs4 import BeautifulSoup
from time import sleep

from trello.exceptions import MovedURL, HorriblyGoneWrongError


def get_soup(url):
    """Method to get the soup from the url"""
    counter = 5
    timeout_counter = 5
    headers = {
        'User-Agent': 'TrelloUpdater',
        'From': 'anubhav.yadav@gmx.com',
    }
    while True:
        try:
            r = requests.get(url, headers=headers, timeout=10)
        except requests.exceptions.ConnectionError:
            print("There is no network connectivity on this computer, will try again after {} seconds".format(counter))
            counter *= counter
            continue
        except requests.exceptions.Timeout:
            print("There is a problem at the target. Retry again in {} seconds".format(timeout_counter))
            continue
        except requests.exceptions.TooManyRedirects:
            print("There is something wrong with the URL, has the website moved?")
            raise MovedURL("Website has moved?")
        except requests.exceptions.RequestException as e:
            print("Something has horribly gone wrong, contact the developer")
            print(e)
            raise HorriblyGoneWrongError("Something has horribly gone wrong, contact the developer")
        break
        while r.status_code != 200:
            sleep(0.5)
            try:
                r = requests.get(url, headers=headers, timeout=10)
            except requests.exceptions.ConnectionError:
                print("Requests blocked! Lets wait for {} seconds".format(counter))
                counter *= counter
                continue
            except requests.exceptions.Timeout:
                print("There is a problem at the target. Retrying again in {} seconds".format(timeout_counter))
                continue
            except requests.exceptions.TooManyRedirects:
                print("There is somthing wrong with the URL, has the website moved?")
                raise MovedURL("Website has moved")
            except requests.exceptions.RequestException as e:
                print("Something horribly gone wrong, contact the developer")
                print(e)
                raise HorriblyGoneWrongError("Somethign has horribly gone wrong, contact the developer")
            break
        sleep(0.5)
    return BeautifulSoup(r.text, "html.parser")



from pony.orm import Database, PrimaryKey
from pony.orm import Required, Optional
from pony.orm import db_session
from pony.orm import select, delete, commit

from trello.settings import DB_PATH

db = Database()


class Course(db.Entity):
    """
    A class to represent a course
    """
    id = PrimaryKey(int, auto=False)
    title = Required(str, unique=True)
    url = Optional(str, unique=True)
    thumbnail = Optional(str)
    processed = Optional(bool, default=False)

    @staticmethod
    @db_session
    def new(id, title, url='', thumbnail='', processed=False):
        """Return a Course either new or from database"""
        course = select(c for c in Course if c.title == title)[:]
        if course:
            return course[0]
        return Course(id=id, title=title, url=url, thumbnail=thumbnail, processed=processed)

    @staticmethod
    @db_session
    def is_course_new(course):
        """
        Check if the course is new or already in db
        """
        return not course.processed

    @staticmethod
    @db_session
    def add_course(course):
        """Add a course to the database"""
        course.processed = True
        commit()

    @staticmethod
    @db_session
    def count():
        """Get total courses in the database"""
        return len(select(c for c in Course)[:])

    @staticmethod
    @db_session
    def delete_all():
        """Delete all information from the table"""
        delete(c for c in Course)


class Book(db.Entity):
    """
    A class to represent a book
    """
    title = Required(str)
    isbn = Required(str)
    author = Optional(str)
    url = Optional(str)
    avg_rating = Optional(str)
    thumbnail = Optional(str)
    processed = Optional(bool, default=False)

    @staticmethod
    @db_session
    def new(title, isbn, author='', url='', avg_rating='', thumbnail='', processed=False):
        """Return a Course either new or from database"""
        book = select(b for b in Book if b.title == title and b.isbn == isbn)[:]
        if book:
            return book[0]
        return Book(title=title, isbn=isbn, author=author, url=url, avg_rating=avg_rating, thumbnail=thumbnail,
                    processed=processed)

    @db_session
    def is_book_new(book):
        """
        Check if the book is new or already in db
        """
        return not book.processed

    @staticmethod
    @db_session
    def add_book(book):
        """Add a book to the db"""
        book.processed = True
        commit()

    @staticmethod
    @db_session
    def count():
        """Get total books in the database"""
        return len(select(b for b in Book)[:])

    @staticmethod
    @db_session
    def delete_all():
        """Delete all information from the table"""
        delete(b for b in Book)


db.bind(provider='sqlite', filename=DB_PATH, create_db=True)
db.generate_mapping(create_tables=True)



class Provider(object):
    """Blueprint for provider"""
    def __init__(self, url=None, username=None, password=None):
        self.url = url
        self.username = username
        self.password = password

    def get_cards(self):
        """`Provider` should return cards"""
        raise NotImplementedError



import click
from pony.orm import db_session
from trello.trello import Trello
from trello.models import Course, Book
from trello.providers.udemy import Udemy
from trello.providers.goodreads import Goodreads
from trello.settings import *


@click.command()
@click.option('--udemy-user-id', default=None, help='user_id of udemy')
@click.option('--goodreads-user-id', default=None, help='user_id of goodreads')
@click.argument('board_id')
def main(udemy_user_id, goodreads_user_id, board_id):
    if not udemy_user_id and not goodreads_user_id:
        return
    trello = Trello(TRELLO_KEY, TRELLO_TOKEN)
    lists = trello.get_lists(board_id)
    l_courses = None

    if udemy_user_id:
        u = Udemy(udemy_user_id)

        for item in lists:
            if item['name'] == 'Courses':
                l_courses = item['id']
            if item['name'] == 'Books':
                l_books = item['id']

        with db_session:
            for c in u.courses:
                course = Course.new(id=c['id'],
                                    title=c['title'],
                                    url='https://www.udemy.com' + c['url'],
                                    thumbnail=c['image_480x270'])
                if Course.is_course_new(course) and l_courses:
                    card_id = trello.create_card(l_courses, name=course.title, desc=course.url)
                    trello.add_attachment_to_card(card_id, course.thumbnail, name='thumbnail')
                    curriculum = u.get_curriculum(course.id)
                    checklist_id = trello.create_checklist(card_id, name='Content')
                    trello.add_items_to_checklist(checklist_id, curriculum)
                    Course.add_course(course)

    if goodreads_user_id:
        g = Goodreads(GOODREADS_KEY, GOODREADS_SECRET, goodreads_user_id)

        with db_session:
            for b in g.books:
                book = Book.new(title=b['title'],
                                isbn=b['isbn'] or 'NA',
                                author=b['author'],
                                url=b['url'],
                                avg_rating=b['avg_rating'],
                                thumbnail=b['thumbnail'])
                if Book.is_book_new(book):
                    desc = "Title: {}\nISBN: {}\nURL: {}\nAuthor: {}\nAverage Rating: {}".format(book.title, book.isbn,
                                                                                                 book.url, book.author,
                                                                                                 book.avg_rating)
                    if l_books:
                        card_id = trello.create_card(list_id=l_books, name=book.title, desc=desc)
                        trello.add_attachment_to_card(card_id, book.thumbnail, name='thumbnail')
                        Book.add_book(book)


if __name__ == '__main__':
    main()



import requests
import xmltodict
import json

from trello.providers import Provider
from trello.exceptions import ResponseNotOKError
from trello.utility import get_soup

class Goodreads(Provider):
    """
    Class to represent the Goodreads books
    """
    def __init__(self, key, secret, username):
        """
        Constructor for the Goodreads class
        """
        self.key = key
        self.secret = secret
        self.username = username
        self.url = "https://www.goodreads.com/user/show/{}.xml".format(self.username)
        self.shelves = []
        self.books = []
        self.get_cards()

    def get_shelves(self):
        """Method to get the shelves of the user"""
        payload = {'key': self.key}
        headers = {
            'User-Agent': 'TrelloUpdater',
            'From': 'anubhav.yadav@gmx.com',
        }
        r = requests.get(self.url, headers=headers, params=payload)
        if r.status_code != 200:
            raise ResponseNotOKError("Response was no 200!: {}".format(r.status_code))
        xml_obj = xmltodict.parse(r.text)
        json_response = json.loads(json.dumps(xml_obj))
        for shelf in json_response['GoodreadsResponse']['user']['user_shelves']['user_shelf']: 
            self.shelves.append({'name': shelf['name'], 'id': shelf['id']['#text']})

    def get_books_from_shelf(self, shelf_name):
        """Method to get list of all books from a given shelf"""
        url = "https://www.goodreads.com/review/list/{}?&shelf={}&per_page=infinite".format(self.username, shelf_name)
        soup = get_soup(url)
        books = []
        for row in soup.find('tbody', {'id': 'booksBody'}).find_all('tr'):
            title = row.find('td', {'class': 'field title'}).find('a').text.strip('\n').strip()
            url = row.find('td', {'class': 'field title'}).find('a').get('href')
            url = "https://goodreads.com{}".format(url)
            isbn = row.find('td', {'class': 'field isbn'}).find('div').text.strip()
            author = row.find('td', {'class': 'field author'}).find('div').text.strip()
            thumbnail = row.find('td', {'class': 'field cover'}).find('img').get('src')
            avg_rating = row.find('td', {'class': 'field avg_rating'}).find('div').text.strip()
            books.append({
                'title': title,
                'url': url,
                'isbn': isbn,
                'author': author,
                'thumbnail': thumbnail,
                'avg_rating': avg_rating
            })
        return books

    def get_cards(self):
        self.get_shelves()
        for shelf in self.shelves:
            self.books.extend(self.get_books_from_shelf(shelf['name']))



from setuptools import setup, find_packages
import pathlib

HERE = pathlib.Path(__file__).parent

README = (HERE / "README.md").read_text()

setup(
    name="python_trello",
    version="0.0.1",
    description="A simple library for easily creating and updating Trello cards.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/anubhavcodes/pytrello",
    author="Anubhav Y",
    author_email="contact+code@anubhav.codes",
    zip_safe=False,
    packages=find_packages(exclude=['tests']),
    include_package_data=True,
    setup_requires=['pytest-runner'],
    install_requires=[
        "requests",
        "bs4",
    ],
)
